@@ -6,6 +6,7 @@ use warnings;
 no autovivification;
 
 use List::Util qw( first );
+use List::AllUtils qw( firstidx );
 use Biber::Config;
 use Biber::Utils;
 use Biber::Constants;
@@ -29,7 +30,8 @@ my $logger = Log::Log4perl::get_logger('main');
     Initialize a Biber::DataModel object
     We are passing in an array of datamodels as there may be more than one in tool
     mode - the one from biber-tool.conf and modifications in a user .conf
-    Later takes precedence.
+    We first merge these before extracting data. In case of conflicts, user .conf
+    datamodel takes precedence.
 
 =cut
 
@@ -38,152 +40,225 @@ sub new {
   my $dms = shift;
   my $self;
   $self = bless {}, $class;
-#  use Data::Dump;dd($dm);exit 0;
+  # use Data::Dump;dd($dms);exit 0;
 
-  # Early check for fatal datamodel errors
+  # Merge global and any user data model
+  my $dm = $dms->[0];
 
-  foreach my $dm ($dms->@*) {
-    # Make sure dates are named *date. A lot of code relies on this.
-    foreach my $date (grep {$_->{datatype} eq 'date'} $dm->{fields}{field}->@*) {
-      unless ($date->{content} =~ m/date$/) {
-        biber_error("Fatal datamodel error: date field '" . $date->{content} . "' must end with string 'date'");
-      }
-    }
+  # If requested, throw away default data model and use user-defined
+  if (Biber::Config->getoption('no_default_datamodel')) {
+    $dm = $dms->[1];
+  }
+  else { # we want to add/modify the default datamodel using the user-supplied subset
+    if (my $udm = $dms->[1]) {
 
-    # Pull out legal entrytypes, fields and constraints and make lookup hash
-    # for quick tests later
-    foreach my $f ($dm->{fields}{field}->@*) {
-
-      # In case of conflicts, we need to remove the previous definitions since
-      # later overrides earlier
-      if (my $previous = $self->{fieldsbyname}{$f->{content}}) {
-
-        if ($f->{format}) {
-          $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{$previous->{format}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{$previous->{format}}->@*;
+      # Constants
+      foreach my $uc ($udm->{constants}{constant}->@*) {
+        my $uce = firstidx {fc($_->{name}) eq fc($uc->{name})} $dm->{constants}{constant}->@*;
+        if ($uce >= 0) { # since constants are named, we can overwrite easily
+          $dm->{constants}{constant}[$uce] = $uc;
         }
-        $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{'*'}->@* = grep {$_ ne $f->{content}} $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{'*'}->@*;
-        $self->{fieldsbyfieldtype}{$previous->{fieldtype}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbyfieldtype}{$previous->{fieldtype}}->@*;
-        $self->{fieldsbydatatype}{$previous->{datatype}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbydatatype}{$previous->{datatype}}->@*;
-        $self->{fieldsbyformat}{$previous->{'format'}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbyformat}{$previous->{format}}->@*;
-        delete $self->{fieldsbyname}{$f->{content}};
-      }
-
-      $self->{fieldsbyname}{$f->{content}} = {'fieldtype'   => $f->{fieldtype},
-                                              'datatype'    => $f->{datatype},
-                                              'format'      => $f->{format} || 'default'};
-      if ($f->{format}) {
-        push $self->{fieldsbytype}{$f->{fieldtype}}{$f->{datatype}}{$f->{format}}->@*, $f->{content};
-      }
-      push $self->{fieldsbytype}{$f->{fieldtype}}{$f->{datatype}}{'*'}->@*, $f->{content};
-      push $self->{fieldsbyfieldtype}{$f->{fieldtype}}->@*, $f->{content};
-      push $self->{fieldsbydatatype}{$f->{datatype}}->@*, $f->{content};
-      push $self->{fieldsbyformat}{$f->{format} || 'default'}->@*, $f->{content};
-
-      # check null_ok
-      if ($f->{nullok}) {
-        $self->{fieldsbyname}{$f->{content}}{nullok} = 1;
-      }
-      # check skips - fields we don't want to output to .bbl
-      if ($f->{skip_output}) {
-        $self->{fieldsbyname}{$f->{content}}{skipout} = 1;
-      }
-    }
-
-    my $constants;
-    foreach my $constant ($dm->{constants}{constant}->@*) {
-      $self->{constants}{$constant->{name}}{type} = $constant->{type};
-      $self->{constants}{$constant->{name}}{value} = $constant->{content};
-    }
-
-    # Get entrytpes from existing model (meaning, already processed XML model)
-    # or XML model. We need to do this in case we have two datamodel sources like
-    # a default tool mode config and a user config
-    foreach my $et (($dm->{entrytypes}{entrytype}->@*, keys $self->{entrytypesbyname}->%*)) {
-      my $es;
-      if (ref($et) eq 'HASH') { # from new config
-        $es = $et->{content};
-
-        # Skip output flag for certain entrytypes
-        if ($et->{skip_output}) {
-          $self->{entrytypesbyname}{$es}{skipout} = 1;
+        else {
+          push $dm->{constants}{constant}->@*, $uc;
         }
       }
-      else {
-        $es = $et; # from existing config
+
+      # Constraints
+      foreach my $uc ($udm->{constraints}[0]{constraint}->@*) {
+        push $dm->{constraints}[0]{constraint}->@*, $uc;
       }
 
-      # fields for entrytypes
-      foreach my $ef ($dm->{entryfields}->@*) {
-        # Found a section describing legal fields for entrytype
-        if (not exists($ef->{entrytype}) or
-            grep {$_->{content} eq $es} $ef->{entrytype}->@*) {
-          foreach my $f ($ef->{field}->@*) {
-            $self->{entrytypesbyname}{$es}{legal_fields}{$f->{content}} = 1;
+      # Entryfields
+      foreach my $uef ($udm->{entryfields}->@*) {
+        if (my $et = $uef->{entrytype}) {
+          my $ef = firstidx {$_->{entrytype}[0]{content} and (fc($_->{entrytype}[0]{content}) eq fc($et->[0]{content}))} $dm->{entryfields}->@*;
+          if ($ef >= 0) {       # Push fields onto existing type
+            push $dm->{entryfields}[$ef]{field}->@*, $uef->{field}->@*;
+          }
+          else {                # Unknown type, create new array member
+            push $dm->{entryfields}->@*, $uef;
+          }
+        }
+        else {                  # general fields for all entrytypes
+          my $ef = firstidx {not exists($_->{entrytype})} $dm->{entryfields}->@*;
+          if ($ef >= 0) {
+            push $dm->{entryfields}[$ef]{field}->@*, $uef->{field}->@*;
           }
         }
       }
 
-      # constraints
-      foreach my $cd ($dm->{constraints}->@*) {
-        # Found a section describing constraints for entrytype
-        if (not exists($cd->{entrytype}) or
-            grep {$_->{content} eq $es} $cd->{entrytype}->@*) {
-          foreach my $c ($cd->{constraint}->@*) {
-            if ($c->{type} eq 'mandatory') {
-              # field
-              foreach my $f ($c->{field}->@*) {
-                push $self->{entrytypesbyname}{$es}{constraints}{mandatory}->@*, $f->{content};
-              }
-              # xor set of fields
-              # [ XOR, field1, field2, ... , fieldn ]
-              foreach my $fxor ($c->{fieldxor}->@*) {
-                my $xorset;
-                foreach my $f ($fxor->{field}->@*) {
-                  push $xorset->@*, $f->{content};
-                }
-                unshift $xorset->@*, 'XOR';
-                push $self->{entrytypesbyname}{$es}{constraints}{mandatory}->@*, $xorset;
-              }
-              # or set of fields
-              # [ OR, field1, field2, ... , fieldn ]
-              foreach my $for ($c->{fieldor}->@*) {
-                my $orset;
-                foreach my $f ($for->{field}->@*) {
-                  push $orset->@*, $f->{content};
-                }
-                unshift $orset->@*, 'OR';
-                push $self->{entrytypesbyname}{$es}{constraints}{mandatory}->@*, $orset;
-              }
+      # Entrytypes
+      foreach my $et ($udm->{entrytypes}{entrytype}->@*) {
+        push $dm->{entrytypes}{entrytype}->@*, $et;
+      }
+
+      # Fields
+      foreach my $f ($udm->{fields}{field}->@*) {
+        my $df = firstidx {fc($_->{content}) eq fc($f->{content}) } $dm->{fields}{field}->@*;
+        if ($df >= 0) {
+          $dm->{fields}{field}->[$df] = $f;
+        }
+        else {
+          push $dm->{fields}{field}->@*, $f;
+        }
+      }
+
+      # Multiscriptfields
+      foreach my $f ($udm->{multiscriptfields}{field}->@*) {
+        push $dm->{multiscriptfields}{field}->@*, $f;
+      }
+    }
+  }
+
+  # First, we normalise all entrytypes and fields to case-folded form for internal
+  # comparisons but we save a map of case-folded variants to actual names
+  # so that we can recover the information later for output
+  foreach my $et ($dm->{entrytypes}{entrytype}->@*) {
+    $self->{casemap}{foldtoorig}{fc($et->{content})} = $et->{content};
+    $et->{content} = fc($et->{content});
+  }
+  foreach my $f ($dm->{fields}{field}->@*) {
+    $self->{casemap}{foldtoorig}{fc($f->{content})} = $f->{content};
+    $f->{content} = fc($f->{content});
+  }
+
+  # Early check for fatal datamodel errors
+  # Make sure dates are named *date. A lot of code relies on this.
+  foreach my $date (grep {$_->{datatype} eq 'date'} $dm->{fields}{field}->@*) {
+    unless ($date->{content} =~ m/date$/) {
+      biber_error("Fatal datamodel error: date field '" . $date->{content} . "' must end with string 'date'");
+    }
+  }
+
+  # Multiscript enabled fields
+  foreach my $f ($dm->{multiscriptfields}{field}->@*) {
+    $self->{multiscriptfields}{$f} = 1;
+  }
+
+  # Pull out legal entrytypes, fields and constraints and make lookup hash
+  # for quick tests later
+  foreach my $f ($dm->{fields}{field}->@*) {
+
+    # In case of conflicts, we need to remove the previous definitions since
+    # later overrides earlier
+    if (my $previous = $self->{fieldsbyname}{$f->{content}}) {
+
+      if ($f->{format}) {
+        $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{$previous->{format}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{$previous->{format}}->@*;
+      }
+      $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{'*'}->@* = grep {$_ ne $f->{content}} $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{'*'}->@*;
+      $self->{fieldsbyfieldtype}{$previous->{fieldtype}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbyfieldtype}{$previous->{fieldtype}}->@*;
+      $self->{fieldsbydatatype}{$previous->{datatype}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbydatatype}{$previous->{datatype}}->@*;
+      $self->{fieldsbyformat}{$previous->{format}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbyformat}{$previous->{format}}->@*;
+      delete $self->{fieldsbyname}{$f->{content}};
+    }
+
+    $self->{fieldsbyname}{$f->{content}} = {'fieldtype'   => $f->{fieldtype},
+                                            'datatype'    => $f->{datatype},
+                                            'format'      => $f->{format} || 'default'};
+    if ($f->{format}) {
+      push $self->{fieldsbytype}{$f->{fieldtype}}{$f->{datatype}}{$f->{format}}->@*, $f->{content};
+    }
+    push $self->{fieldsbytype}{$f->{fieldtype}}{$f->{datatype}}{'*'}->@*, $f->{content};
+    push $self->{fieldsbyfieldtype}{$f->{fieldtype}}->@*, $f->{content};
+    push $self->{fieldsbydatatype}{$f->{datatype}}->@*, $f->{content};
+    push $self->{fieldsbyformat}{$f->{format} || 'default'}->@*, $f->{content};
+
+    # check null_ok
+    if ($f->{nullok}) {
+      $self->{fieldsbyname}{$f->{content}}{nullok} = 1;
+    }
+    # check skips - fields we don't want to output to .bbl
+    if ($f->{skip_output}) {
+      $self->{fieldsbyname}{$f->{content}}{skipout} = 1;
+    }
+  }
+
+  my $constants;
+  foreach my $constant ($dm->{constants}{constant}->@*) {
+    $self->{constants}{$constant->{name}}{type} = $constant->{type};
+    $self->{constants}{$constant->{name}}{value} = $constant->{content};
+  }
+
+  foreach my $et ($dm->{entrytypes}{entrytype}->@*) {
+    my $es = $et->{content};
+
+    # Skip output flag for certain entrytypes
+    if ($et->{skip_output}) {
+      $self->{entrytypesbyname}->{$es}{skipout} = 1;
+    }
+    # fields for entrytypes
+    foreach my $ef ($dm->{entryfields}->@*) {
+      # Found a section describing legal fields for entrytype
+      if (not exists($ef->{entrytype}) or
+          grep {$_->{content} eq $es} $ef->{entrytype}->@*) {
+        foreach my $f ($ef->{field}->@*) {
+          $self->{entrytypesbyname}{$es}{legal_fields}{$f->{content}} = 1;
+        }
+      }
+    }
+
+    # constraints
+    foreach my $cd ($dm->{constraints}->@*) {
+      # Found a section describing constraints for entrytype
+      if (not exists($cd->{entrytype}) or
+          grep {$_->{content} eq $es} $cd->{entrytype}->@*) {
+        foreach my $c ($cd->{constraint}->@*) {
+          if ($c->{type} eq 'mandatory') {
+            # field
+            foreach my $f ($c->{field}->@*) {
+              push $self->{entrytypesbyname}{$es}{constraints}{mandatory}->@*, $f->{content};
             }
-            # Conditional constraints
-            # [ ANTECEDENT_QUANTIFIER
-            #   [ ANTECEDENT LIST ]
-            #   CONSEQUENT_QUANTIFIER
-            #   [ CONSEQUENT LIST ]
-            # ]
-            elsif ($c->{type} eq 'conditional') {
-              my $cond;
-              $cond->[0] = $c->{antecedent}{quant};
-              $cond->[1] = [ map { $_->{content} } $c->{antecedent}{field}->@* ];
-              $cond->[2] = $c->{consequent}{quant};
-              $cond->[3] = [ map { $_->{content} } $c->{consequent}{field}->@* ];
-              push $self->{entrytypesbyname}{$es}{constraints}{conditional}->@*, $cond;
+            # xor set of fields
+            # [ XOR, field1, field2, ... , fieldn ]
+            foreach my $fxor ($c->{fieldxor}->@*) {
+              my $xorset;
+              foreach my $f ($fxor->{field}->@*) {
+                push $xorset->@*, $f->{content};
+              }
+              unshift $xorset->@*, 'XOR';
+              push $self->{entrytypesbyname}{$es}{constraints}{mandatory}->@*, $xorset;
             }
-            # data constraints
-            elsif ($c->{type} eq 'data') {
-              my $data;
-              $data->{fields} = [ map { $_->{content} } $c->{field}->@* ];
-              $data->{datatype} = $c->{datatype};
-              $data->{rangemin} = $c->{rangemin};
-              $data->{rangemax} = $c->{rangemax};
-              $data->{pattern} = $c->{pattern};
-              push $self->{entrytypesbyname}{$es}{constraints}{data}->@*, $data;
+            # or set of fields
+            # [ OR, field1, field2, ... , fieldn ]
+            foreach my $for ($c->{fieldor}->@*) {
+              my $orset;
+              foreach my $f ($for->{field}->@*) {
+                push $orset->@*, $f->{content};
+              }
+              unshift $orset->@*, 'OR';
+              push $self->{entrytypesbyname}{$es}{constraints}{mandatory}->@*, $orset;
             }
           }
+          # Conditional constraints
+          # [ ANTECEDENT_QUANTIFIER
+          #   [ ANTECEDENT LIST ]
+          #   CONSEQUENT_QUANTIFIER
+          #   [ CONSEQUENT LIST ]
+          # ]
+          elsif ($c->{type} eq 'conditional') {
+            my $cond;
+            $cond->[0] = $c->{antecedent}{quant};
+            $cond->[1] = [ map { $_->{content} } $c->{antecedent}{field}->@* ];
+            $cond->[2] = $c->{consequent}{quant};
+            $cond->[3] = [ map { $_->{content} } $c->{consequent}{field}->@* ];
+            push $self->{entrytypesbyname}{$es}{constraints}{conditional}->@*, $cond;
+          }
+          # data constraints
+          elsif ($c->{type} eq 'data') {
+            my $data;
+            $data->{fields} = [ map { $_->{content} } $c->{field}->@* ];
+            $data->{datatype} = $c->{datatype};
+            $data->{rangemin} = $c->{rangemin};
+            $data->{rangemax} = $c->{rangemax};
+            $data->{pattern} = $c->{pattern};
+            push $self->{entrytypesbyname}{$es}{constraints}{data}->@*, $data;
+          }
         }
       }
     }
   }
+
   # Calculate and store some convenient lists of DM fields. This is to save the expense
   # of constructing these in dense loops like entry processing/output.
   # Mostly only used for .bbl output since that's the most commonly used one and so
@@ -267,6 +342,17 @@ sub new {
   return $self;
 }
 
+=head2 get_outcase
+
+    Returns the original datamodel field/entrytype case for output
+
+=cut
+
+sub get_outcase {
+  my ($self, $string) = @_;
+  return $self->{casemap}{foldtoorig}{$string};
+}
+
 =head2 constants
 
     Returns array ref of constant names
@@ -305,6 +391,16 @@ sub get_constant_value {
   }
 }
 
+=head2 is_multiscript
+
+    Returns boolean to say if a field is a multiscript field
+
+=cut
+
+sub is_multiscript {
+  my ($self, $field) = shift;
+  return $self->{multiscriptfields}{$field} ? 1 : 0;
+}
 
 =head2 fieldtypes
 
@@ -969,6 +1065,11 @@ sub generate_bltxml_schema {
           $writer->startTag('zeroOrMore');# for example, XDATA doesn't need a name
           $writer->startTag('element', 'name' => "$bltx:names");
 
+          $writer->startTag('choice');
+          # xdata attribute ref
+          $writer->emptyTag('ref', 'name' => 'xdata');
+
+          $writer->startTag('group');
           # useprefix attribute
           $writer->comment('useprefix option');
           $writer->startTag('optional');
@@ -1006,6 +1107,11 @@ sub generate_bltxml_schema {
           # Individual name element
           $writer->startTag('element', 'name' => "$bltx:name");
 
+          $writer->startTag('choice');
+          # xdata attribute ref
+          $writer->emptyTag('ref', 'name' => 'xdata');
+
+          $writer->startTag('group');
           # useprefix attribute
           $writer->comment('useprefix option');
           $writer->startTag('optional');
@@ -1051,8 +1157,12 @@ sub generate_bltxml_schema {
           $writer->endTag();    # choice
           $writer->endTag();    # namepart element
           $writer->endTag();    # oneOrMore
+          $writer->endTag();    # group
+          $writer->endTag();    # choice
           $writer->endTag();    # name element
           $writer->endTag();    # oneOrMore
+          $writer->endTag();    # group
+          $writer->endTag();    # choice
           $writer->endTag();    # names element
           $writer->endTag();    # zeroOrMore
           # ========================
@@ -1064,17 +1174,22 @@ sub generate_bltxml_schema {
           foreach my $list ($dm->get_fields_of_type($ft, $dt)->@*) {
             $writer->startTag('optional');
             $writer->startTag('element', 'name' => "$bltx:$list");
-
+            $writer->startTag('choice');
+            $writer->emptyTag('ref', 'name' => 'xdata');
             $writer->startTag('choice');
             $writer->emptyTag('text');# text
             $writer->startTag('element', 'name' => "$bltx:list");
             $writer->startTag('oneOrMore');
             $writer->startTag('element', 'name' => "$bltx:item");
+            $writer->startTag('choice');
+            $writer->emptyTag('ref', 'name' => 'xdata');
             $writer->emptyTag('text');# text
+            $writer->endTag(); # choice
             $writer->endTag(); # item element
             $writer->endTag(); # oneOrMore element
             $writer->endTag(); # list element
             $writer->endTag(); # choice
+            $writer->endTag(); # choice
             $writer->endTag(); # $list element
             $writer->endTag(); # optional
           }
@@ -1088,7 +1203,10 @@ sub generate_bltxml_schema {
           foreach my $field ($dm->get_fields_of_type($ft, $dt)->@*) {
             $writer->startTag('optional');
             $writer->startTag('element', 'name' => "$bltx:$field");
+            $writer->startTag('choice');
+            $writer->emptyTag('ref', 'name' => 'xdata');
             $writer->emptyTag('data', 'type' => 'anyURI');
+            $writer->endTag();   # choice
             $writer->endTag();   # $field element
             $writer->endTag();# optional
           }
@@ -1103,9 +1221,18 @@ sub generate_bltxml_schema {
             $writer->startTag('optional');
             $writer->startTag('element', 'name' => "$bltx:$field");
 
+            $writer->startTag('choice');
+            # xdata attribute ref
+            $writer->emptyTag('ref', 'name' => 'xdata');
+
             $writer->startTag('element', 'name' => "$bltx:list");
             $writer->startTag('oneOrMore');
             $writer->startTag('element', 'name' => "$bltx:item");
+
+            $writer->startTag('choice');
+            # xdata attribute ref
+            $writer->emptyTag('ref', 'name' => 'xdata');
+            $writer->startTag('group');
             $writer->startTag('element', 'name' => "$bltx:start");
             $writer->emptyTag('text');
             $writer->endTag();  # start element
@@ -1115,11 +1242,14 @@ sub generate_bltxml_schema {
             $writer->emptyTag('empty');
             $writer->endTag();  # choice
             $writer->endTag();  # end element
-            $writer->endTag(); # item element
-            $writer->endTag();   # oneOrMore element
+            $writer->endTag();  # group
+            $writer->endTag();  # choice
+            $writer->endTag();  # item element
+            $writer->endTag();  # oneOrMore element
             $writer->endTag();  # list element
-            $writer->endTag();   # $field element
-            $writer->endTag();# optional
+            $writer->endTag();  # choice
+            $writer->endTag();  # $field element
+            $writer->endTag();  # optional
           }
           $writer->endTag();# interleave
           # ==============================
@@ -1152,16 +1282,21 @@ sub generate_bltxml_schema {
             else {
               $writer->startTag('element', 'name' => "$bltx:$field");
               $writer->startTag('choice');
+              $writer->emptyTag('ref', 'name' => 'xdata');
+              $writer->startTag('choice');
               $writer->startTag('list');
               $writer->startTag('oneOrMore');
               $writer->emptyTag('data', 'type' => 'string');
               $writer->endTag(); # oneOrMore
-              $writer->endTag();    # list
+              $writer->endTag(); # list
+              $writer->startTag('element', 'name' => "$bltx:list");
               $writer->startTag('oneOrMore');
-              $writer->startTag('element', 'name' => "$bltx:key");
+              $writer->startTag('element', 'name' => "$bltx:item");
               $writer->emptyTag('text');# text
-              $writer->endTag(); # key element
+              $writer->endTag(); # item element
               $writer->endTag(); # oneOrMore
+              $writer->endTag(); # list element
+              $writer->endTag(); # choice
               $writer->endTag(); # choice
               $writer->endTag(); # $field element
             }
@@ -1213,7 +1348,10 @@ sub generate_bltxml_schema {
           foreach my $field ($dm->get_fields_of_type($ft, $dt)->@*) {
             $writer->startTag('optional');
             $writer->startTag('element', 'name' => "$bltx:$field");
+            $writer->startTag('choice');
+            $writer->emptyTag('ref', 'name' => 'xdata');
             $writer->emptyTag('text');# text
+            $writer->endTag(); # choice
             $writer->endTag(); # $field element
             $writer->endTag();# optional
           }
@@ -1225,6 +1363,18 @@ sub generate_bltxml_schema {
     }
   }
 
+  # xdata attribute definition
+  # ===========================
+  $writer->comment('xdata attribute definition');
+  $writer->startTag('define', 'name' => 'xdata');
+  $writer->startTag('optional');
+  $writer->startTag('attribute', 'name' => 'xdata');
+  $writer->emptyTag('text');# text
+  $writer->endTag();# attribute
+  $writer->endTag();# optional
+  $writer->endTag();# define
+  # ===========================
+
   # gender attribute definition
   # ===========================
   $writer->comment('gender attribute definition');
@@ -1834,7 +1984,6 @@ __END__
 
 =head1 AUTHORS
 
-François Charette, C<< <firmicus at ankabut.net> >>
 Philip Kime C<< <philip at kime.org.uk> >>
 
 =head1 BUGS
@@ -1844,7 +1993,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2019 François Charette and Philip Kime, all rights reserved.
+Copyright 2012-2020 Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
